/*
 * ELF payload decoding support.
 *
 * Copyright (C) 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/util/elf.h>

#include "elf.h"



/* Program segment construction for 32- or 64-bit variants. */

template <typename PROGRAM_HEADER>
ProgramSegmentVariant<PROGRAM_HEADER>::ProgramSegmentVariant(PROGRAM_HEADER *header)
: _header(header)
{
  init();
}

template <typename PROGRAM_HEADER>
bool ProgramSegmentVariant<PROGRAM_HEADER>::loadable()
{
  return _header->p_type == PT_LOAD;
}

template <typename PROGRAM_HEADER>
offset_t ProgramSegmentVariant<PROGRAM_HEADER>::file_contents()
{
  return _header->p_filesz;
}

template <typename PROGRAM_HEADER>
offset_t ProgramSegmentVariant<PROGRAM_HEADER>::file_offset()
{
  return _header->p_offset;
}

template <typename PROGRAM_HEADER>
l4_addr_t ProgramSegmentVariant<PROGRAM_HEADER>::region_address()
{
  return _header->p_vaddr;
}

template <typename PROGRAM_HEADER>
offset_t ProgramSegmentVariant<PROGRAM_HEADER>::region_size()
{
  return _header->p_memsz;
}

template <typename PROGRAM_HEADER>
l4re_rm_flags_t ProgramSegmentVariant<PROGRAM_HEADER>::region_flags()
{
  l4re_rm_flags_t flags = 0;

  if (_header->p_flags & PF_R)
    flags |= L4RE_RM_F_R;
  if (_header->p_flags & PF_W)
    flags |= L4RE_RM_F_W;
  if (_header->p_flags & PF_X)
    flags |= L4RE_RM_F_X;

  return flags;
}

  

/* Generic payload destruction. */

Payload::~Payload()
{
  if (_segments != NULL)
  {
    for (unsigned int i = 0; i < segments(); i++)
      delete _segments[i];
  }
}

unsigned int Payload::segments()
{
  return 0;
}



/* ELF payload construction for 32- or 64-bit variants. */

template <typename HEADER, typename PROGRAM_HEADER>
PayloadVariant<HEADER, PROGRAM_HEADER>::PayloadVariant(HEADER *header)
: _header(header)
{
}

template <typename HEADER, typename PROGRAM_HEADER>
l4_addr_t PayloadVariant<HEADER, PROGRAM_HEADER>::entry_point()
{
  return _header->e_entry;
}

template <typename HEADER, typename PROGRAM_HEADER>
offset_t PayloadVariant<HEADER, PROGRAM_HEADER>::header_extent()
{
  return sizeof(HEADER);
}

template <typename HEADER, typename PROGRAM_HEADER>
offset_t PayloadVariant<HEADER, PROGRAM_HEADER>::program_header_extent()
{
  return _header->e_phoff + _header->e_phentsize * _header->e_phnum;
}

template <typename HEADER, typename PROGRAM_HEADER>
unsigned int PayloadVariant<HEADER, PROGRAM_HEADER>::segments()
{
  return _header->e_phnum;
}

template <typename HEADER, typename PROGRAM_HEADER>
Segment *PayloadVariant<HEADER, PROGRAM_HEADER>::segment(unsigned int i)
{
  if (i >= segments())
    return NULL;

  if (_segments == NULL)
  {
    _segments = new Segment *[segments()];

    for (unsigned int j = 0; j < segments(); j++)
      _segments[j] = NULL;
  }

  Segment *segment = _segments[i];

  if (segment == NULL)
  {
    segment = new ProgramSegmentVariant<PROGRAM_HEADER>(
                    (PROGRAM_HEADER *) ((char *) _header + _header->e_phoff +
                    _header->e_phentsize * i));
    _segments[i] = segment;
  }

  return segment;
}



/* Return the appropriate payload object. */

Payload *get_payload(char *buf)
{
  switch (buf[EI_CLASS])
  {
    case (char) 1:
      return new PayloadVariant<Elf32_Ehdr, Elf32_Phdr>((Elf32_Ehdr *) buf);
    case (char) 2:
      return new PayloadVariant<Elf64_Ehdr, Elf64_Phdr>((Elf64_Ehdr *) buf);
    default:
      return NULL;
  }
}

/* vim: tabstop=2 expandtab shiftwidth=2
*/
